// cc-by 3.0 Austria, (c) 2010 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
import javafx.stage.Stage;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.KeyCode;
import javafx.scene.transform.Transform;
import javafx.animation.Timeline;
import javafx.animation.KeyFrame;
import javafx.animation.Interpolator;
import java.lang.IllegalArgumentException;
import controller.GameLogic;
import model.GameState;
import model.Direction.*;
import view.FieldNode;
import view.StatusViewNode;
import model.Score;

var game: GameLogic = GameLogic {};
var fieldNode: FieldNode = FieldNode { field: game.field; };
var statusGroup = StatusViewNode {
            x: 50
            y: 50

            score: bind game.score

            startAction: game.togglePause
            getScoreAction: Score.getScores
            sendScoreAction: Score.setScore
        }
var gameState: GameState = bind game.state on replace {
            if (gameState == GameState.RUNNING) {
                statusGroup.visible = false;
                sceneGroup.requestFocus();
            } else if (gameState == GameState.PAUSED) {
                statusGroup.visible = true;
                statusGroup.requestFocus();
                statusGroup.statusText = "Paused.\nPress Space to Continue!";
            } else if (gameState == GameState.GAMEOVER) {
                statusGroup.visible = true;
                statusGroup.statusText = "Game Over! Your Score: {game.score}\nPress Space to Restart!";
            } else {
                throw new IllegalArgumentException("Unknown game state");
            }
        };
var scaleTransform: Transform = bind Transform.scale(scene.width / fieldNode.WIDTH, scene.height / fieldNode.HEIGHT);
var rotateAngle: Number = 0;
def pivotX = bind scene.width / 2;
def pivotY = bind scene.height / 2;
var rotateTransform: Transform = bind Transform.rotate(rotateAngle, pivotX, pivotY);
var sceneGroup: Group = Group {
            transforms: bind [rotateTransform, scaleTransform]
            content: [fieldNode, statusGroup]
            onKeyPressed: handleKeyEvent
        }
var scene: Scene = Scene {
            content: sceneGroup
        };
var stage: Stage = Stage {
            title: "Worm";
            width: 600;
            height: 600;
            scene: scene;
        };
var focused = bind sceneGroup.focused on replace {
            if (not focused and game.state != GameState.GAMEOVER) {
                game.pause();
            }
        }
var stageFocus = bind stage.containsFocus on replace {
            if (stageFocus) {
                sceneGroup.requestFocus();
            }
        }
def rotateAnim: Timeline = Timeline {
            repeatCount: Timeline.INDEFINITE
            keyFrames: [
                KeyFrame {
                    time: 0s
                    values: rotateAngle => 0;
                },
                KeyFrame {
                    time: 20s
                    values: rotateAngle => 360 tween Interpolator.LINEAR;
                }
            ]
        }

function handleKeyEvent(evt: KeyEvent): Void {
    if (evt.code == KeyCode.VK_UP) {
        game.setWormDirection(UP);
    } else if (evt.code == KeyCode.VK_DOWN) {
        game.setWormDirection(DOWN);
    } else if (evt.code == KeyCode.VK_LEFT) {
        game.setWormDirection(LEFT);
    } else if (evt.code == KeyCode.VK_RIGHT) {
        game.setWormDirection(RIGHT);
    } else if (evt.code == KeyCode.VK_ENTER and evt.altDown) {
        stage.fullScreen = not stage.fullScreen;
    } else if (evt.code == KeyCode.VK_P or evt.code == KeyCode.VK_SPACE) {
        game.togglePause();
    } else if (evt.code == KeyCode.VK_R) {
        toggleRotate();
    } else {
        // nothing to do
        return;
    }
};

function toggleRotate() {
    if (not rotateAnim.running or rotateAnim.paused) {
        rotateAnim.play();
    } else {
        rotateAnim.pause();
        rotateAngle = 0;
        rotateAnim.time = 0s;
    }
}

