// cc-by 3.0 Austria, (c) 2010 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
package controller;

import model.GameState;
import model.Worm;
import model.Field;
import model.Goody;
import model.Direction;
import <<at>>.jku.ssw.worms.score.crypto.Cryptor;
import javafx.animation.Timeline;
import javafx.animation.KeyFrame;
import java.lang.IllegalArgumentException;
import java.util.Random;

public class GameLogic {

    def worm: Worm = Worm {
                startX: Field.COLUMNS / 2;
                startY: Field.ROWS / 2;
            };
    public-read var state: GameState;
    public-read var score: Integer = bind sizeof worm.segments;
    def goody: Goody = Goody {};
    public-read def field: Field = Field {
                worm: worm;
                goody: goody;
            };
    def random: Random = new Random();
    def anim: Timeline = Timeline {
                repeatCount: Timeline.INDEFINITE
                keyFrames: [
                    KeyFrame {
                        time: 40ms
                        canSkip: true
                        action: function() {
                            worm.move();
                            checkGame();
                            plantGoody();
                        }
                    }
                ]
            }

    function checkGame(): Void {
        if (isWormOnWall() or isWormOnWorm()) {
            handleGameOver();
        } else if (isWormOnGoody()) {
            worm.grow(goody.value);
            goody.consume();
        }
    }

    function handleGameOver(): Void {
        anim.stop();
        state = GameState.GAMEOVER;
    }

    function plantGoody() {
        if (not goody.available) {
            goody.plant(random.nextInt(Field.COLUMNS), random.nextInt(Field.ROWS), goody.value + 1);
        }
    }

    function isWormOnWall(): Boolean {
        var seg = worm.segments[0];
        return seg.x < 0 or seg.y < 0 or seg.x >= Field.COLUMNS or seg.y >= Field.ROWS;
    }

    function isWormOnWorm(): Boolean {
        var head = worm.segments[0];
        for (body in worm.segments[1..]) {
            if (head.x == body.x and head.y == body.y) {
                return true;
            }
        }
        return false;
    }

    function isWormOnGoody(): Boolean {
        var head = worm.segments[0];
        return goody.available and goody.x == head.x and goody.y == head.y;
    }

    public function togglePause(): Void {
        if (state == GameState.GAMEOVER) {
            start();
        } else if (state == GameState.RUNNING) {
            pause();
        } else if (state == GameState.PAUSED) {
            resume();
        } else {
            throw new IllegalArgumentException("Unknown game state");
        }
    }

    public function start(): Void {
        worm.reset();
        goody.reset();
        resume();
    }

    public function pause(): Void {
        anim.pause();
        state = GameState.PAUSED;
    }

    function resume(): Void {
        state = GameState.RUNNING;
        anim.play();
    }

    public function setWormDirection(dir: Direction): Void {
        if (worm.direction != dir.oposite()) {
            worm.direction = dir
        }
    }

    init {
        pause();
    }

}
