// cc-by 3.0 Austria, (c) 2010 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
package view;

import model.Score;
import javafx.scene.CustomNode;
import javafx.scene.Group;
import javafx.scene.shape.Rectangle;
import javafx.scene.paint.Color;
import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Stop;
import javafx.scene.text.Text;
import javafx.scene.text.Font;
import javafx.scene.control.Button;
import javafx.scene.control.TextBox;
import javafx.scene.control.Label;
import javafx.scene.layout.VBox;
import javafx.scene.layout.HBox;
import javafx.geometry.VPos;
import javafx.scene.transform.Transform;
import javafx.geometry.HPos;

public class StatusViewNode extends CustomNode {

    def WIDTH: Number = 500;
    def HEIGHT: Number = 500;
    public var statusText: String;
    public var x: Number;
    public var y: Number;
    public var score: Integer;
    public var startAction: function();
    public var getScoreAction: function(function(Score[]));
    public var sendScoreAction: function(Score, function(Score[]));
    var scores: Score[];
    def scoreFont = Font { size: 20 }
    def scoreColor = Color.ANTIQUEWHITE;
    def highScoreValues = VBox {
                nodeHPos: HPos.LEFT;
                content: bind for (s: Score in scores) {
                    Text {
                        content: "{s.score}"
                        fill: scoreColor;
                        font: scoreFont
                    }
                }
            }
    def highScoreNames = VBox {
                nodeHPos: HPos.LEFT;
                content: bind for (s: Score in scores) {
                    Text {
                        content: "{s.name}"
                        fill: scoreColor;
                        font: scoreFont
                    }
                }
            }
    def updateScoreButton = Button {
                text: "Refresh Scores"
                visible: bind getScoreAction != null
                action: refreshScores;
            }

    function refreshScores(): Void {
        if (getScoreAction != null) {
            getScoreAction(updateScores);
        }
    }

    function updateScores(newScores: Score[]): Void {
        scores = newScores;
    }

    def getHighScore = VBox {
                content: HBox {
                    nodeVPos: VPos.TOP;
                    spacing: 10
                    content: bind if (sizeof scores == 0) {
                        [
                            updateScoreButton,
                            Text {
                                content: "No Highscore Available..."
                                font: scoreFont
                                fill: scoreColor
                            }
                        ]
                    } else {
                        [updateScoreButton, highScoreValues, highScoreNames]
                    }
                }
            }
    def sendScoreButton = Button {
                text: "Send Score"
                action: function() {
                    if (sendScoreAction != null) {
                        def s = Score { name: nameBox.text.trim(), score: score };
                        sendScoreAction(s, updateScores);
                    }
                }
            }
    def nameBox = TextBox {
                columns: 12;
                selectOnFocus: true;
                font: Font { size: 20 };
            }
    def nameLabel = Label {
                text: "Name"
                font: Font { size: 24 }
                textFill: Color.ANTIQUEWHITE
            }
    def setHighScore = HBox {
                spacing: 10
                nodeVPos: VPos.CENTER
                visible: bind sendScoreAction != null
                content: [nameLabel, nameBox, sendScoreButton]
            }
    def playButton = Button {
                text: "Space ;-)"
                action: startAction
            }
    def status = Text {
                content: bind statusText;
                font: Font { size: 24 }
                fill: Color.ANTIQUEWHITE
            }
    def foreground = VBox {
                spacing: 10
                layoutX: 30
                layoutY: 30
                content: [
                    status,
                    playButton,
                    Rectangle { height: 20, width: 0 },
                    setHighScore,
                    Rectangle { height: 20, width: 0 },
                    getHighScore
                ]
            }
    def gradient = LinearGradient {
                startX: 0.0, startY: 0.0, endX: 1.0, endY: 1.0
                proportional: true
                stops: [
                    Stop {
                        offset: 0.0
                        color: Color { red: 0, green: 0, blue: 0, opacity: 0.925 }
                    },
                    Stop {
                        offset: 1.0
                        color: Color { red: .2, green: .2, blue: .2, opacity: .750 }
                    }
                ]
            }
    def background = Rectangle {
                x: 0; y: 0;
                width: WIDTH; height: HEIGHT;
                arcWidth: 20; arcHeight: 20;
                fill: gradient
            }
    def move = bind Transform.translate(x, y);

    public override function create() {
        Group {
            transforms: [move]
            content: [background, foreground]
        }
    }

    public override function requestFocus() {
        playButton.requestFocus();
    }

    postinit {
        refreshScores();
    }

}

